@@ -1,13 +1,12 @@
 # This file is a part of pyctr.
 #
-# Copyright (c) 2017-2021 Ian Burgwin
+# Copyright (c) 2017-2022 Ian Burgwin
 # This file is licensed under The MIT License (MIT).
 # You can find the full license text in LICENSE in the root of this project.
 
 """Module for interacting with CTR Importable Archive (CIA) files."""
 
 from enum import IntEnum
-from io import BytesIO
 from threading import Lock
 from typing import TYPE_CHECKING, NamedTuple
 
@@ -136,6 +135,14 @@ class CIAReader(TypeReaderCryptoBase):
         # store case-insensitivity for RomFSReader
         self._case_insensitive = case_insensitive
 
+        # this contains the location of each section, as well as the IV of encrypted ones
+        self.sections = {}
+
+        self.contents = {}
+
+        active_contents_tmd = set()
+        self.content_info = []
+
         header = self._file.read(0x20)
 
         archive_header_size = readle(header[0x0:0x4])
@@ -178,9 +185,6 @@ class CIAReader(TypeReaderCryptoBase):
         # lazy method to get the total size
         self.total_size = meta_offset + meta_size
 
-        # this contains the location of each section, as well as the IV of encrypted ones
-        self.sections = {}
-
         def add_region(section: 'Union[int, CIASection]', offset: int, size: int, iv: 'Optional[bytes]'):
             region = CIARegion(section=section, offset=offset, size=size, iv=iv)
             self.sections[section] = region
@@ -194,21 +198,16 @@ class CIAReader(TypeReaderCryptoBase):
             add_region(CIASection.Meta, meta_offset, meta_size, None)
 
         # this will load the titlekey to decrypt the contents
-        self._file.seek(self._start + ticket_offset)
-        ticket = self._file.read(ticket_size)
-        self._crypto.load_from_ticket(ticket)
+        with self.open_raw_section(CIASection.Ticket) as ticket:
+            self._crypto.load_from_ticket(ticket.read())
 
         # the tmd describes the contents: ID, index, size, and hash
-        self._file.seek(self._start + tmd_offset)
-        tmd_data = self._file.read(tmd_size)
-        self.tmd = TitleMetadataReader.load(BytesIO(tmd_data))
+        with self.open_raw_section(CIASection.TitleMetadata) as tmd:
+            self.tmd = TitleMetadataReader.load(tmd)
 
         if seed:
             add_seed(self.tmd.title_id, seed)
 
-        active_contents_tmd = set()
-        self.content_info = []
-
         # this does a first check to make sure there are no missing contents that are marked active in content_index
         for record in self.tmd.chunk_records:
             if record.cindex in active_contents:
@@ -220,8 +219,6 @@ class CIAReader(TypeReaderCryptoBase):
         if active_contents ^ active_contents_tmd:
             raise InvalidCIAError('Missing active contents in the TMD')
 
-        self.contents = {}
-
         # this goes through the contents and figures out their regions, then creates an NCCHReader
         curr_offset = content_offset
         for record in self.content_info:
